import zipfile
from tempfile import NamedTemporaryFile

import simplebot
from deltachat import Message
from emoji import demojize
from simplebot.bot import Replies

from simplebot_stickers import signal

__version__ = "1.0.0"


@simplebot.filter
def filter_messages(message, replies, bot) -> None:
    """Process messages with sticker packs URL or images."""
    if not message.chat.is_group():
        if message.filemime.startswith("image/"):
            replies.add(filename=message.filename, viewtype="sticker")
        if signal.is_pack(message.text):
            pack_name, stickers = signal.get_stickers(message.text)
            with NamedTemporaryFile(
                dir=bot.account.get_blobdir(),
                prefix=pack_name + "-",
                suffix=".stickers.zip",
                delete=False,
            ) as file:
                zip_path = file.name
            with zipfile.ZipFile(
                zip_path, "w", compression=zipfile.ZIP_DEFLATED
            ) as zfile:
                for sticker in stickers:
                    name = "{}.{}+{}.webp".format(
                        sticker.id,
                        demojize(sticker.emoji, delimiters=("", "")),
                        sticker.emoji,
                    )
                    zfile.writestr(name, sticker.image_data)
            replies.add(filename=zip_path, quote=message)


@simplebot.command
def sticker(message: Message, replies: Replies) -> None:
    """Send attached or quoted image as sticker."""
    file = None
    if message.filemime.startswith("image/"):
        file = message.filename
    elif message.quote and message.quote.filemime.startswith("image/"):
        file = message.quote.filename
    if file:
        replies.add(filename=file, viewtype="sticker")



"""Signal sticker pack downlad tools"""

from urllib import parse

import anyio
from signalstickers_client import StickersClient


async def _get_stickers(pack_id: str, pack_key: str) -> tuple:
    async with StickersClient() as client:
        pack = await client.get_pack(pack_id, pack_key)
    return (pack.title, pack.stickers)


def get_stickers(url: str) -> tuple:
    result = parse.urlsplit(url)
    if result.query:
        data = dict(parse.parse_qsl(result.query))
    else:
        data = dict(parse.parse_qsl(result.fragment))
    return anyio.run(_get_stickers, data["pack_id"], data["pack_key"])


def is_pack(url: str) -> bool:
    return url.startswith(("https://signal.art/addstickers/", "sgnl://addstickers/"))



"""Setup module installation."""

import os
import re

from setuptools import find_packages, setup

if __name__ == "__main__":
    MODULE_NAME = "simplebot_stickers"
    DESC = "Sticker packs for all your Delta Chat needs (SimpleBot plugin)"
    URL = "https://github.com/simplebot-org/simplebot_stickers"

    with open(os.path.join(MODULE_NAME, "__init__.py")) as fh:
        version = re.search(r"__version__ = \"(.*?)\"", fh.read(), re.M).group(1)

    with open("README.rst") as fh:
        long_description = fh.read()
    with open("CHANGELOG.rst") as fh:
        long_description += "\n" + fh.read()

    with open("requirements.txt", encoding="utf-8") as req:
        install_requires = [
            line.replace("==", ">=")
            for line in req.read().split("\n")
            if line and not line.startswith(("#", "-"))
        ]
    with open("requirements-test.txt", encoding="utf-8") as req:
        test_deps = [
            line.replace("==", ">=")
            for line in req.read().split("\n")
            if line and not line.startswith(("#", "-"))
        ]

    setup(
        name=MODULE_NAME,
        version=version,
        description=DESC,
        long_description=long_description,
        long_description_content_type="text/x-rst",
        author="The SimpleBot Contributors",
        author_email="adbenitez@nauta.cu",
        url=URL,
        keywords="simplebot plugin deltachat",
        license="MPL",
        classifiers=[
            "Development Status :: 4 - Beta",
            "Environment :: Plugins",
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
            "Operating System :: OS Independent",
            "Topic :: Utilities",
        ],
        zip_safe=False,
        include_package_data=True,
        packages=find_packages(),
        install_requires=install_requires,
        extras_require={"test": test_deps},
        entry_points={
            "simplebot.plugins": "{0} = {0}".format(MODULE_NAME),
        },
    )
